#
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""CI Automation branch helpers."""

import hashlib
import os
import subprocess
import re


def get_revno(url):
    '''Get the current revision for a branch URL.

    If the URL contains a fixed revno, return it.'''

    match = re.search(';revno=([0-9]*)', url)
    if match:
        # The revno is fixed. Return it.
        return match.group(1)

    out = subprocess.check_output(['bzr', 'revno', url],
                                  stderr=subprocess.DEVNULL)
    return out.decode('utf8').strip('\n')


def get_path_revno(url, path):
    '''Return the last change revision of the path in the branch URL.'''

    if ';revno=' in url:
        # The revno is fixed. Return it.
        return get_revno(url)

    p = '{}/{}'.format(url, path)
    out = subprocess.check_output(['bzr', 'log', '-S', '-l', '1', p],
                                  stderr=subprocess.DEVNULL)
    out = out.decode('utf8')
    match = re.search(' *([0-9]*)', out)
    if match:
        return match.group(1)
    else:
        return None


def file_contents_on_branch(branch, stage, path):
    '''Get the contents of the given file at the given branch.

    Returns empty-string if the file does not exist and cannot be accessed.
    '''

    if not branch.endswith('/'):
        branch += '/'
    full_path = branch + os.path.join(stage, path)
    try:
        out = subprocess.check_output(['bzr', 'cat', full_path],
                                      stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        return ''
    return out.decode('utf8').strip('\n')


def branches_in_contents(contents):
    '''Parse out the branches contained in the given file and return a
    generator of them.'''

    for line in contents.split('\n'):
        # Ignore comments
        if line.startswith('#'):
            continue
        match = re.search('\s((?:lp:|bzr\+ssh:|http:)\S*)', line)
        if match:
            yield match.group(1)


def get_identifier(branch, stage):
    """Return a set of information about the spec branch and its contents.

    Returns a tuple container the textual identification information and
    a corresponding MD5 hash of its contents.
    """
    identifiers = []

    # Cover changes in stage (name & revno)
    stage_revno = get_path_revno(branch, os.path.dirname(stage))
    identifiers.append("{}\t{}".format(stage, stage_revno))

    # Cover changes in collected branches (url/path and revno),
    # considers stage-specific and top-level 'collect' files.
    for stage in (stage, os.path.dirname(stage)):
        contents = file_contents_on_branch(branch, stage, 'collect')
        for b in branches_in_contents(contents):
            identifiers.append("{}\t{}".format(b, get_revno(b)))

    info = '\n'.join(identifiers)

    return info, hashlib.md5(info.encode('utf-8')).hexdigest()
