# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""CI Automation: juju helpers"""

import os
import shutil
import subprocess
import yaml

from . import utils


def destroy_environment(name, base):
    # We can safely destroy the existing environment, as the services are
    # promised to be idempotent.

    env_dir = os.path.join(base, name)
    env = os.environ.copy()
    env['JUJU_HOME'] = env_dir

    try:
        args = ['juju', 'status']
        subprocess.check_call(
            args, env=env,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL
        )
    except subprocess.CalledProcessError:
        # No juju environment here.
        pass
    else:
        # FIXME or leave alone?
        args = ['juju', 'destroy-environment', '-y', '--force', name]
        subprocess.check_call(args, env=env)

    # XXX cprov 20150402: also destroy related bits secgroups,

    shutil.rmtree(env_dir)


def new_environment(name, base, devel=False, network=None):
    env_dir = os.path.join(base, name)
    env = os.environ.copy()
    env['JUJU_HOME'] = env_dir

    if os.path.exists(env_dir):
        destroy_environment(name, base)

    os.makedirs(env_dir)
    contents = {
        'default': name,
        'environments': {
            name: {
                'type': 'openstack',
                'control-bucket': 'juju-{}'.format(name),
                # If we ever remove this, we'll need to manually
                # set security group rules allowing ssh from
                # 91.189.90.53/32. Juju creates these rules for
                # 0.0.0.0, which juju-reaper removes (would
                # allow public SSH for any instance with a
                # floating IP).
                'use-default-secgroup': 'true',
            },
        },
    }

    if network:
        contents['environments'][name]['network'] = network

    if not devel:
        # Otherwise we chew up Prodstack's disk IO
        # upgrading, but it doesn't work for 'bootstack' (our devel).
        contents['environments'][name]['image-stream'] = 'daily'

    with open(os.path.join(env_dir, 'environments.yaml'), 'w') as fp:
        fp.write(yaml.dump(contents, default_flow_style=False))

    args = ['juju', 'bootstrap']
    utils.check_call(args, env=env)

    return env_dir
