#!/usr/bin/env python3
#
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""CI Automation tests for branch helpers."""

import os
import shutil
import subprocess
import sys
import tempfile
import unittest


HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.append(os.path.join(HERE, '../..'))

from ci_automation.branch import (
    get_identifier,
    get_path_revno,
    get_revno,
)


class TestBranchUtils(unittest.TestCase):

    def setUp(self):
        super(TestBranchUtils, self).setUp()
        self.branch = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.branch)
        self.branch_init()

    def branch_init(self):
        subprocess.check_call(['bzr', 'init'], cwd=self.branch,
                              stderr=subprocess.DEVNULL,
                              stdout=subprocess.DEVNULL)

    def branch_add(self, path):
        subprocess.check_call(['bzr', 'add', path],
                              cwd=self.branch,
                              stderr=subprocess.DEVNULL,
                              stdout=subprocess.DEVNULL)

    def branch_commit(self, message='Bazinga!'):
        subprocess.check_call(['bzr', 'commit', '-m', message],
                              cwd=self.branch,
                              stderr=subprocess.DEVNULL,
                              stdout=subprocess.DEVNULL)

    def branch_add_stage(self, name):
        stage = os.path.join(self.branch, name)
        os.makedirs(stage)
        with open(os.path.join(stage, 'collect'), 'w') as fd:
            fd.write('foo lp:foo;revno=1')
        self.branch_add(stage)
        self.branch_commit()

    def branch_update_collect(self, stage, content='foo lp:foo;revno=1'):
        collect = os.path.join(self.branch, stage, 'collect')
        with open(collect, 'w') as fd:
            fd.write(content)
        self.branch_commit()

    def test_get_revno(self):
        self.assertEqual(get_revno(self.branch), '0')

    def test_get_revno_with_embedded(self):
        self.assertEqual(get_revno('lp:fake;revno=33'), '33')

    def test_get_revno_with_path(self):
        paths = (os.path.join(self.branch, 'bob'),
                 os.path.join(self.branch, 'bob2'))
        for path in paths:
            with open(path, 'w'):
                pass
            self.branch_add(path)
            self.branch_commit()
        self.assertEqual(get_path_revno(self.branch, 'bob'), '1')
        self.assertEqual(get_path_revno(self.branch, 'bob2'), '2')

    def test_get_identifier_other_stage(self):
        # Creating other stage does not change the original identifier.
        stage = 'bob/devel'
        self.branch_add_stage(stage)
        self.assertEqual(
            ('\n'.join(['bob/devel\t1',
                        'lp:foo;revno=1\t1']),
             '90945bed4b6e27d500ae1ae05c7bba1c'),
            get_identifier(self.branch, stage))
        self.branch_add_stage('the-builder/devel')
        self.assertEqual(
            ('\n'.join(['bob/devel\t1',
                        'lp:foo;revno=1\t1']),
             '90945bed4b6e27d500ae1ae05c7bba1c'),
            get_identifier(self.branch, stage))

    def test_get_identifier_spec_changes(self):
        # Modifying spec contents results in identifier changes.
        stage = 'bob/devel'
        self.branch_add_stage(stage)
        self.assertEqual(
            ('\n'.join(['bob/devel\t1',
                        'lp:foo;revno=1\t1']),
             '90945bed4b6e27d500ae1ae05c7bba1c'),
            get_identifier(self.branch, stage))
        repo_path = os.path.join(self.branch, stage, 'repo')
        with open(repo_path, 'w'):
            pass
        self.branch_add(stage)
        self.branch_commit()
        self.assertEqual(
            ('\n'.join(['bob/devel\t2',
                        'lp:foo;revno=1\t1']),
             'dce1f2c4a1a1634f49a4572fa8630765'),
            get_identifier(self.branch, stage))

    def test_get_identifier_collect_changes(self):
        # Modifying collect contents results in identifier changes.
        stage = 'bob/devel'
        self.branch_add_stage(stage)
        self.branch_update_collect(stage, 'foo lp:foo;revno=7')
        self.assertEqual(
            ('\n'.join(['bob/devel\t2',
                        'lp:foo;revno=7\t7']),
             'f4bdf7cb82d6f1a9fc725d1bf70eef75'),
            get_identifier(self.branch, stage))
        self.branch_update_collect(
            stage, '\n'.join(['foo lp:~cprov/foo;revno=7',
                             'bar lp:bar;revno=5']))
        self.assertEqual(
            ('\n'.join(['bob/devel\t3',
                        'lp:~cprov/foo;revno=7\t7',
                        'lp:bar;revno=5\t5']),
             '261a6c824fa28680e7e0869194a12152'),
            get_identifier(self.branch, stage))


if __name__ == '__main__':
    unittest.main()
