#!/usr/bin/env python3
#
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""CI Automation: mojo spec deployer."""


import argparse
import configparser
import os
import subprocess
import sys


from ci_automation import (
    branch,
    juju,
    mojo,
    utils,
)


SERIES = 'trusty'


def fix_permissions(project):
    ''' "Why do I get permission denied?" from
    https://mojo.canonical.com/readme.html'''

    utils.check_call(['sudo', 'chmod', '755', '/var/lib/lxc'])
    path = '{}.{}'.format(project, SERIES)
    path = os.path.join('/var/lib/lxc', path)
    utils.check_call(['sudo', 'chmod', '755', path])


def create_service_conf(project, stage, **kwargs):
    c = configparser.ConfigParser()

    c['adt'] = {
        'max_retries': 3,
        'image_flavor': kwargs.get('adt_image_flavor'),
        'test_branch': 'lp:snappy/selftest',
    }

    c['image'] = {
        'channel': 'ubuntu-core/devel-proposed',
        'device': 'generic_amd64',
        'location': '/tmp/latest-core-image-version',
        'poll_period': '60'
    }

    c['swift'] = {
        'public_url': kwargs.get('swift_public_url') or '<not-set>',
    }

    c['image_mapper'] = {
        'url': kwargs.get('image_mapper_url') or '<not-set>',
    }

    c['logstash'] = {
        'host': kwargs.get('logstash_host') or '127.0.0.1',
        'port': 5959,
        'version': 1,
    }

    c['amqp'] = {
        'uris': kwargs.get(
            'amqp_uris') or 'amqp://guest:guest@localhost:5672//'
    }

    c['nova'] = {
        'os_username': kwargs.get('os_username') or os.environ['OS_USERNAME'],
        'os_tenant_name': kwargs.get(
            'os_tenant_name') or os.environ['OS_TENANT_NAME'],
        'os_password': kwargs.get('os_password') or os.environ['OS_PASSWORD'],
        'os_auth_url': kwargs.get('os_auth_url') or os.environ['OS_AUTH_URL'],
        'os_region_name': kwargs.get(
            'os_region_name') or os.environ['OS_REGION_NAME'],
        'extra_args': kwargs.get('adt_extra_args') or '',
    }

    # This is an ugly hack to keep things going for adt-cloud and
    # core-image solutions under the same tenant.
    filename = (
        'adt.conf' if 'adt' in stage else 'core-service.conf')

    path = '/srv/mojo/LOCAL/{}/{}/{}'.format(project, stage, filename)
    os.makedirs(os.path.dirname(path), exist_ok=True)
    with open(path, 'w') as fp:
        c.write(fp)


def main():
    desc = 'Deploy a given mojo spec in isolation.'
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('--project', default='mojo-stg-ue-ci-engineering')
    parser.add_argument('--stage', required=True)
    parser.add_argument('--branch', required=True)
    parser.add_argument('--devel', action="store_true", default=False)
    parser.add_argument('--base')
    parser.add_argument('--network')
    parser.add_argument('--amqp-uris')
    parser.add_argument('--logstash-host')
    parser.add_argument('--swift-public-url')
    parser.add_argument('--image-mapper-url')
    parser.add_argument('--os-username')
    parser.add_argument('--os-tenant-name')
    parser.add_argument('--os-password')
    parser.add_argument('--os-auth-url')
    parser.add_argument('--os-region-name')
    parser.add_argument('--nova-extra-args')
    args = parser.parse_args()

    base = args.base or '/srv/juju-environments'

    if not os.path.exists(base):
        print('Please create {}.'.format(base))
        return 1

    if not mojo.project_exists(args.project):
        msg = 'Please run `sudo mojo project-new --series trusty {}`.'
        msg = msg.format(args.project)
        print(msg)
        return 1

    # Le sigh.
    if args.devel:
        fix_permissions(args.project)

    # Relies on the canonical-mojo-specs layout, e.g.:
    # ue/mojo-ue-adt-cloud-worker/devel
    spec_name = args.stage.split('/')[1]

    computed_nova_extra_args = ' '.join([
        args.nova_extra_args or '',
        '--net-id={}'.format(args.network) if args.devel else '',
    ]).strip()
    computed_adt_image_flavor = (
        'cpu2-ram2-disk10' if not args.devel else 'm1.smaller')

    create_service_conf(
        args.project,
        args.stage,
        network=args.network,
        amqp_uris=args.amqp_uris,
        logstash_host=args.logstash_host,
        swift_public_url=args.swift_public_url,
        image_mapper_url=args.image_mapper_url,
        os_username=args.os_username,
        os_tenant_name=args.os_tenant_name,
        os_password=args.os_password,
        os_auth_url=args.os_auth_url,
        os_region_name=args.os_region_name,
        nova_extra_args=computed_nova_extra_args,
        adt_image_flavor=computed_adt_image_flavor,
    )

    # Juju environment & mojo workspaces have identical names. Names
    # are unique by appending a spec content identifier including
    # all collected branch URLs and their current revision numbers, e.g.:
    # mojo-ue-core-result-checker-c745311e948bd62221eafdcfcc552edf
    info, identifier = branch.get_identifier(args.branch, args.stage)
    env_name = '{}-{}'.format(spec_name, identifier)

    # For our default production deployments, we want to use the
    # special golive manifest always
    manifest = 'manifest' if args.devel else 'manifest-golive'

    try:
        mojo.new_workspace(args.project, args.stage, args.branch, env_name)
        env_dir = juju.new_environment(
            env_name, base, devel=args.devel, network=args.network)
        mojo.deploy(args.project, args.stage, args.branch, manifest, env_dir)
    except utils.BetterCalledProcessError as err:
        print(err)
        return 1

    print('Deployed environment: {}'.format(env_dir))

    return 0


if __name__ == '__main__':
    sys.exit(main())
